local ALT_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\altsIcon.tga"
local OFFLINE_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\offlineIcon.tga"
local KEYLESS_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\keylessIcon.tga"
local ANNOUNCE_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\announceIcon.tga"
local COMPLETE_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\completeIcon.tga"
local SCHEDULE_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\scheduleIcon.tga"
local OPPOSING_FACTION_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\opposingFactionIcon.tga"
local LOW_LEVEL_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\levelingIcon.tga"
local ALT_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\altsIconDisabled.tga"
local OFFLINE_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\offlineIconDisabled.tga"
local KEYLESS_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\keylessIconDisabled.tga"
local ANNOUNCE_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\announceIconDisabled.tga"
local COMPLETE_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\completeIconDisabled.tga"
local OPPOSING_FACTION_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\opposingFactionIconDisabled.tga"
local LOW_LEVEL_DISABLED_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\levelingIconDisabled.tga"
local GOLD_STAR_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\goldStar.tga"
local MAX_LEVEL = 120
local characterListings = {}

local function CreateSpacer(width)
	local label = KeystoneRollCall.AceGUI:Create('Label');
	label:SetText(" ");
	if width <= 0 then
		label:SetFullWidth(true)
	else
		label:SetWidth(width)
	end
	return label
end

function KeystoneRollCall:Hide()
	if self.frame and self.frame:IsVisible() then
		self.frame:Hide()
	end
end

function KeystoneRollCall:RefreshTab()

	local tab = self.tabs[self.activeTab]
	if tab.isKeyTable then
		for k,v in pairs(self.tableData) do
		  self.tableData[k] = nil
		end
		KeystoneRollCall.filtered = 0;
		wipe(characterListings)
		tab.populate();
		
		table.sort(self.tableData,function(ra,rb)
			return self:CompareKeystones(ra[1],rb[1])
		end)
		self.scrollTable:SortData();
		self:SetStatus();
	end
end

function KeystoneRollCall:AddListing(keystone,online)
	if not keystone or characterListings[keystone.name] then
		return
	end
	
	local tab = self.tabs[self.activeTab]
	local includeKeyless = tab.propertyShowKeyless and KeystoneRollCall.db.global[tab.propertyShowKeyless]
	local includeAlts = tab.propertyShowAlts and KeystoneRollCall.db.global[tab.propertyShowAlts]
	local includeOffline = tab.propertyShowOffline and KeystoneRollCall.db.global[tab.propertyShowOffline]
	local includeComplete = tab.propertyShowComplete and KeystoneRollCall.db.global[tab.propertyShowComplete]
	local includeLeveling = tab.propertyShowLeveling and KeystoneRollCall.db.global[tab.propertyShowLeveling]
	local includeOpposingFaction = tab.propertyShowOpposingFaction and KeystoneRollCall.db.global[tab.propertyShowOpposingFaction]
	
	if not online and not includeOffline then
		return
	end
	
	local dataRow;
	local nameParts =  {strsplit("-",keystone.name)};
	if keystone.keystoneLevel and keystone.keystoneLevel >0 or includeKeyless then
		if includeLeveling or (tonumber(self:GetCharacterData(keystone.name).level) or MAX_LEVEL) >= MAX_LEVEL then
			if includeOpposingFaction or KCLib:GetPlayerKeystone().faction == keystone.faction then
				if includeComplete or not keystone.bestKeystoneLevel or keystone.bestKeystoneLevel < KeystoneRollCall.MAX_REWARD_LEVEL then
					if online then
						dataRow =  {
							keystone,
							tab.propertyShowAlts and " " or KeystoneRollCall:colorFaction(nameParts[2],keystone.faction),
							KeystoneRollCall:colorClass(nameParts[1],keystone.class),
							KeystoneRollCall:colorClass(KRCLocal:Get(keystone.class),keystone.class),
							KeystoneRollCall:getIlvlLine(keystone),
							keystone.seasonScore and tonumber(keystone.seasonScore)  > 0 and keystone.seasonScore or " ",
							keystone.keystoneLevel == 0 and "" or ("+"..KeystoneRollCall:colorLevelDifficulty(keystone.keystoneLevel,keystone.keystoneLevel)),
							keystone.dummy and KeystoneRollCall:colorOffline(KRCLocal:Get("error_addon_not_found")) or KeystoneRollCall:getDungeonName(keystone.keystoneDungeonId),
							KeystoneRollCall:getBestRunWithIlvlLine(keystone.bestKeystoneLevel, keystone.bestKeystoneDungeonId)
						}
					else
						dataRow =  {
							keystone,
							tab.propertyShowAlts and " " or KeystoneRollCall:colorOffline(nameParts[2]),
							KeystoneRollCall:colorOffline(nameParts[1]),
							KeystoneRollCall:colorOffline(KRCLocal:Get(keystone.class)),
							KeystoneRollCall:colorOffline(KeystoneRollCall:getIlvlLine(keystone)),
							KeystoneRollCall:colorOffline(keystone.seasonScore and tonumber(keystone.seasonScore)  > 0 and keystone.seasonScore or " "),
							keystone.keystoneLevel == 0 and "" or KeystoneRollCall:colorOffline("+"..keystone.keystoneLevel),
							KeystoneRollCall:colorOffline(keystone.dummy and KRCLocal:Get("error_addon_not_found") or KeystoneRollCall:getDungeonName(keystone.keystoneDungeonId)),
							KeystoneRollCall:getBestRunWithIlvlLineOffline(keystone.bestKeystoneLevel, keystone.bestKeystoneDungeonId)
						}
					end
					if dataRow and KeystoneRollCall:DoSearch(dataRow) then
						tinsert(KeystoneRollCall.tableData, dataRow);
						characterListings[keystone.name] = dataRow
					end
				end	
			end
		end
	end
	if online and includeAlts then
		KeystoneRollCall:ForEachAltKeystoneEntry(keystone.name,function(altKeystone,mainsKeystone)
			if characterListings[altKeystone.name] then
				if online then
					characterListings[altKeystone.name][2] = KeystoneRollCall:colorClass(nameParts[1],mainsKeystone.class)
				else
					characterListings[altKeystone.name][2] = KeystoneRollCall:colorOffline(nameParts[1])
				end
			elseif altKeystone.name and (includeKeyless or altKeystone and altKeystone.keystoneLevel and altKeystone.keystoneLevel > 0) then
				if includeLeveling or (tonumber(self:GetCharacterData(altKeystone.name).level) or MAX_LEVEL) >= MAX_LEVEL then
					if includeOpposingFaction or KCLib:GetPlayerKeystone().faction == altKeystone.faction then
						if includeComplete or not altKeystone.bestKeystoneLevel or altKeystone.bestKeystoneLevel < KeystoneRollCall.MAX_REWARD_LEVEL then
							local altNameParts =  {strsplit("-",altKeystone.name)};
							local dataRowAlt;
							if online then
								dataRowAlt = {
									altKeystone,
									KeystoneRollCall:colorClass(nameParts[1],mainsKeystone.class),
									KeystoneRollCall:colorClass(altNameParts[1],altKeystone.class),
									KeystoneRollCall:colorClass(KRCLocal:Get(altKeystone.class),altKeystone.class),
									KeystoneRollCall:getIlvlLine(altKeystone),
									altKeystone.seasonScore and tonumber(altKeystone.seasonScore)  > 0 and altKeystone.seasonScore or " ",
									altKeystone.keystoneLevel == 0 and "" or ("+"..KeystoneRollCall:colorLevelDifficulty(altKeystone.keystoneLevel,altKeystone.keystoneLevel)),
									KeystoneRollCall:getDungeonName(altKeystone.keystoneDungeonId),
									KeystoneRollCall:getBestRunWithIlvlLine(altKeystone.bestKeystoneLevel, altKeystone.bestKeystoneDungeonId)
								}
							else
								dataRowAlt = {
									altKeystone,
									KeystoneRollCall:colorOffline(nameParts[1]),
									KeystoneRollCall:colorOffline(altNameParts[1]),
									KeystoneRollCall:colorOffline(KRCLocal:Get(altKeystone.class)),
									KeystoneRollCall:colorOffline(KeystoneRollCall:getIlvlLine(altKeystone)),
									KeystoneRollCall:colorOffline(altKeystone.seasonScore and tonumber(altKeystone.seasonScore) > 0 and altKeystone.seasonScore or " "),
									altKeystone.keystoneLevel == 0 and "" or KeystoneRollCall:colorOffline("+"..altKeystone.keystoneLevel),
									KeystoneRollCall:colorOffline(KeystoneRollCall:getDungeonName(altKeystone.keystoneDungeonId)),
									KeystoneRollCall:getBestRunWithIlvlLineOffline(altKeystone.bestKeystoneLevel, altKeystone.bestKeystoneDungeonId)
								}
							end
							
							if KeystoneRollCall:DoSearch(dataRowAlt) then
								tinsert(KeystoneRollCall.tableData, dataRowAlt);
								characterListings[altKeystone.name] = dataRow
							end		
						end
					end
				end
			end
		end)
	end
	return dataRow;
end

function KeystoneRollCall:DoSearch(dataRow)
	local searchTerm = self.frame.searchbox:GetText()
	if not searchTerm or searchTerm:len() == 0 then
		return true;
	end
	searchTerm = searchTerm:lower()
	local value
	for i = 2, #dataRow do
		if dataRow[i] and tostring(dataRow[i]) then
			value = string.gsub(tostring(dataRow[i]),"|c(........)(.-)|r",function(a,b)  
				return b;
			end):lower()
			if value and string.find(value, searchTerm, 1, true) then
				return true;
			end
		end
	end
	KeystoneRollCall.filtered = KeystoneRollCall.filtered +1
	return false
end

function KeystoneRollCall:HideFilters()
	local frame = self.frame;
	frame.completeToggleButton:Hide();
	frame.altToggleButton:Hide();
	frame.offlineToggleButton:Hide();
	frame.keylessToggleButton:Hide();
	frame.levelingToggleButton:Hide();
	frame.otherFactionToggleButton:Hide();
	frame.searchbox:Hide();
end

function KeystoneRollCall:SetupFilters(showSearch, completeBtnProperty, altBtnProperty, offlineBtnProperty, keylessBtnProperty, levelingBtnProperty, otherFactionBtnProperty)
	local frame = self.frame;
	local lastElement = frame.completeToggleButton;
	
	frame.completeToggleButton:Show()
	frame.completeToggleButton:SetProperty(completeBtnProperty)
	
	if altBtnProperty then
		frame.altToggleButton:SetProperty(altBtnProperty)
		frame.altToggleButton:SetPoint("TOPRIGHT",lastElement,"TOPLEFT", 1, 0)
		frame.altToggleButton:Show()
		lastElement = frame.altToggleButton
	else
		frame.altToggleButton:Hide()
	end
	
	if offlineBtnProperty then
		frame.offlineToggleButton:SetProperty(offlineBtnProperty)
		frame.offlineToggleButton:SetPoint("TOPRIGHT",lastElement,"TOPLEFT", 1, 0)
		frame.offlineToggleButton:Show()
		lastElement = frame.offlineToggleButton
	else
		frame.offlineToggleButton:Hide()
	end
	
	if keylessBtnProperty then
		frame.keylessToggleButton:SetProperty(keylessBtnProperty)
		frame.keylessToggleButton:SetPoint("TOPRIGHT",lastElement,"TOPLEFT", 1, 0)
		frame.keylessToggleButton:Show()
		lastElement = frame.keylessToggleButton
	else
		frame.keylessToggleButton:Hide()
	end
	
	if levelingBtnProperty then
		frame.levelingToggleButton:SetProperty(levelingBtnProperty)
		frame.levelingToggleButton:SetPoint("TOPRIGHT",lastElement,"TOPLEFT", 1, 0)
		frame.levelingToggleButton:Show()
		lastElement = frame.levelingToggleButton
	else
		frame.levelingToggleButton:Hide()
	end
	
	if otherFactionBtnProperty then
		frame.otherFactionToggleButton:SetProperty(otherFactionBtnProperty)
		frame.otherFactionToggleButton:SetPoint("TOPRIGHT",lastElement,"TOPLEFT", 1, 0)
		frame.otherFactionToggleButton:Show()
		lastElement = frame.otherFactionToggleButton
	else
		frame.otherFactionToggleButton:Hide()
	end
	
	if showSearch then
		frame.searchbox:ClearAllPoints()
		if lastElement then
			frame.searchbox:SetPoint("TOPRIGHT",lastElement,"TOPLEFT", 1, 0)
		else
			frame.searchbox:SetPoint("TOPRIGHT",frame.frame,"TOPRIGHT",-14,-34)
		end
		frame.searchbox:Show()
		lastElement = frame.searchbox
	else
		frame.searchbox:Hide()
	end
end

local function toggleButtonOnClick(self)
	local isOn = not KeystoneRollCall.db.global[self.property]
	KeystoneRollCall.db.global[self.property] = isOn
	self:SetNormalTexture(isOn and self.onIcon or self.offIcon)
	KeystoneRollCall:RefreshTab()
end

local function toggleButtonOnEnter(self)
	GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
	GameTooltip:ClearAllPoints();
	GameTooltip:SetPoint("BOTTOMLEFT",self,"TOPLEFT");
	GameTooltip:SetText(self.tooltip)
	
	if not GameTooltip.bg then
		GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
		GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
		GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
		GameTooltip.bg:SetColorTexture(0,0,0,1)

	end
	GameTooltip.bg:Show()
	GameTooltip:Show()
end

local function toggleButtonOnLeave(box)
	if GameTooltip.bg then
		GameTooltip.bg:Hide()
	end
	GameTooltip:Hide()
end

local function createToggleButton(onIcon,offIcon,tooltip)
	local frame = KeystoneRollCall.frame;
	local btn = CreateFrame("Button", nil, frame.frame,"UIPanelButtonTemplate")
	btn:SetSize(30, 30)
	btn:SetPoint("TOPRIGHT",frame.frame,"TOPRIGHT",-20,-28)
	btn:SetNormalTexture(onIcon)
	btn.onIcon = onIcon
	btn.offIcon = offIcon
	btn.tooltip = tooltip
	
	btn:SetScript("OnClick",toggleButtonOnClick)
	btn:SetScript("OnEnter", toggleButtonOnEnter)
	btn:SetScript("OnLeave", toggleButtonOnLeave)
	btn.SetProperty = function(self,property)
		self.property = property
		local isOn = KeystoneRollCall.db.global[property]
		KeystoneRollCall.db.global[property] = isOn
		self:SetNormalTexture(isOn and self.onIcon or self.offIcon)
	end
	return btn
end

local function UpdateSeasonBestDisplay()
	for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
		local btn = KeystoneRollCall.frame.seasonBestButtons[dungeonId];
		local keystone = KCLib:GetPlayerKeystone();
		local level = keystone and keystone.seasonRunInfos and keystone.seasonRunInfos[dungeonId] and keystone.seasonRunInfos[dungeonId].level or 0;
		btn.levelLabel:SetText(level);
		local score = keystone and keystone.seasonRunInfos and keystone.seasonRunInfos[dungeonId] and keystone.seasonRunInfos[dungeonId].score or 0;
		if score > 0 then
			btn.scoreValueLabel:SetText(math.floor(score*10)/10);
		else
			btn.scoreValueLabel:SetText(" ");
		end
	end
end


local function ApplySettings()
	local frame = KeystoneRollCall.frame;
    frame.announceGroupLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12);
    frame.announceGuildLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12);
    frame.announceTargetLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12);
    frame.closeLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16);
	frame.statusBox:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12);
			
	for _,btn in pairs(frame.seasonBestButtons) do
		btn.levelLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 19, "THICKOUTLINE")
		btn.scoreValueLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "THICKOUTLINE")
	end

	frame.affix1Label:SetFont(KeystoneRollCall.db.global.primaryFontPath, 22)
	frame.affix2Label:SetFont(KeystoneRollCall.db.global.primaryFontPath, 22)
	frame.affix3Label:SetFont(KeystoneRollCall.db.global.primaryFontPath, 22)
	frame.showAffixScheduleLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
	
	if GameTooltip.keystoneElements then
		GameTooltip.keystoneElements.characterName:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16)
		GameTooltip.keystoneElements.guildName:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		GameTooltip.keystoneElements.characterDesc:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		GameTooltip.keystoneElements.ilvlLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		GameTooltip.keystoneElements.seasonScoreLabelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 14)
		GameTooltip.keystoneElements.seasonScoreLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 20)
		GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 14)
		GameTooltip.keystoneElements.accountSeasonScoreLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 20)

		for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
			GameTooltip.keystoneElements["map"..i.."name"]:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
			GameTooltip.keystoneElements["map"..i.."accountname"]:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
			GameTooltip.keystoneElements["map"..i.."level"]:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
			GameTooltip.keystoneElements["map"..i.."accountlevel"]:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
		end

		GameTooltip.keystoneElements.knownAltsLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		GameTooltip.keystoneElements.knownAltsLabelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 14)
		GameTooltip.keystoneElements.lastSeenLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
	end
end

function KeystoneRollCall:Show()
	if self.frame and self.frame:IsVisible() then
		return
	end
	GuildRoster()
	
	if self.frame then
		self.frame:Show()
		self:RefreshTab()
	else
		
		local maps = C_ChallengeMode.GetMapTable();
		table.sort(maps, function(dungeonId1, dungeonId2) 
			local name1 = C_ChallengeMode.GetMapUIInfo(dungeonId1) or " ";
			local name2 = C_ChallengeMode.GetMapUIInfo(dungeonId2) or " ";
			return name1:upper() < name2:upper(); 
		end);
		KeystoneRollCall.sortedMaps = maps;
	
		local frame = self.AceGUI:Create("Window")
		_G["KeystoneRollCall_Display"] = frame
		tinsert(UISpecialFrames, "KeystoneRollCall_Display") 
		self.frame = frame
		frame:SetTitle(KRCLocal:Get("title"))
		frame:SetStatusText("")
		local frameWidth = 760;
		frame:SetWidth(frameWidth)
		frame:SetHeight(610)
		frame:EnableResize(false);
		frame:SetCallback("OnClose", function(widget)  KeystoneRollCall.frame:Hide()  end)
		frame:SetLayout("Flow")
		
		
		frame.top1 = frame.frame:CreateTexture(nil)
		frame.top1:SetColorTexture(.20,.20,.20,1)
		frame.top1:SetPoint("TOP",frame.frame,"TOP",0,-23)
		frame.top1:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.top1:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.top1:SetHeight(1)
		
		
		frame.top2 = frame.frame:CreateTexture(nil)
		frame.top2:SetColorTexture(.45,.45,.45,1)
		frame.top2:SetPoint("TOP",frame.top1,"BOTTOM",0,0)
		frame.top2:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.top2:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.top2:SetHeight(1)
		
		frame.top3 = frame.frame:CreateTexture(nil)
		frame.top3:SetColorTexture(.05,.05,.05,1)
		frame.top3:SetPoint("TOP",frame.top2,"BOTTOM",0,0)
		frame.top3:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.top3:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.top3:SetHeight(1)
		
		frame.scoreBG = frame.frame:CreateTexture(nil, "BACKGROUND")
		frame.scoreBG:SetPoint("TOP",frame.top3,"BOTTOM",0,0)
		frame.scoreBG:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.scoreBG:SetPoint("RIGHT",frame.frame,"RIGHT",-6,0)
		frame.scoreBG:SetColorTexture(.16,.16,.16,1)
		frame.scoreBG:SetHeight(40)
		
		
		frame.divider1 = frame.frame:CreateTexture(nil)
		frame.divider1:SetColorTexture(.05,.05,.05,1)
		frame.divider1:SetPoint("TOP",frame.scoreBG,"BOTTOM",0,0)
		frame.divider1:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.divider1:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.divider1:SetHeight(1)
		
		
		frame.divider2 = frame.frame:CreateTexture(nil)
		frame.divider2:SetColorTexture(.45,.45,.45,1)
		frame.divider2:SetPoint("TOP",frame.divider1,"BOTTOM",0,0)
		frame.divider2:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.divider2:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.divider2:SetHeight(1)
		
		frame.divider3 = frame.frame:CreateTexture(nil)
		frame.divider3:SetColorTexture(.20,.20,.20,1)
		frame.divider3:SetPoint("TOP",frame.divider2,"BOTTOM",0,0)
		frame.divider3:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.divider3:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.divider3:SetHeight(1)
		
		
		
		
		frame.affixBG = frame.frame:CreateTexture(nil, "BACKGROUND")
		frame.affixBG:SetPoint("TOP",frame.divider3,"BOTTOM",0,0)
		frame.affixBG:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.affixBG:SetPoint("RIGHT",frame.frame,"RIGHT",-6,0)
		frame.affixBG:SetColorTexture(.11,.11,.11,1)
		frame.affixBG:SetHeight(42)
		
		frame.divider4 = frame.frame:CreateTexture(nil)
		frame.divider4:SetColorTexture(.20,.20,.20,1)
		frame.divider4:SetPoint("TOP",frame.affixBG,"BOTTOM",0,0)
		frame.divider4:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.divider4:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.divider4:SetHeight(1)
		
		
		frame.divider5 = frame.frame:CreateTexture(nil)
		frame.divider5:SetColorTexture(.45,.45,.45,1)
		frame.divider5:SetPoint("TOP",frame.divider4,"BOTTOM",0,0)
		frame.divider5:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.divider5:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.divider5:SetHeight(1)
		
		frame.divider6 = frame.frame:CreateTexture(nil)
		frame.divider6:SetColorTexture(.05,.05,.05,1)
		frame.divider6:SetPoint("TOP",frame.divider5,"BOTTOM",0,0)
		frame.divider6:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.divider6:SetPoint("RIGHT",frame.frame,"RIGHT",-5,0)
		frame.divider6:SetHeight(1)
		
		frame.bg = frame.frame:CreateTexture(nil, "BACKGROUND")
		frame.bg:SetColorTexture(.07,.07,.07,1)
		frame.bg:SetPoint("TOP",frame.divider6,"BOTTOM",0,0)
		frame.bg:SetPoint("LEFT",frame.frame,"LEFT",8,0)
		frame.bg:SetPoint("RIGHT",frame.frame,"RIGHT",-6,0)
		frame.bg:SetPoint("BOTTOM",frame.frame,"BOTTOM",0,8)
		
		
		frame:AddChild(CreateSpacer(0));
		frame:AddChild(CreateSpacer(0));
		frame:AddChild(CreateSpacer(0));
		frame:AddChild(CreateSpacer(0));
		frame:AddChild(CreateSpacer(0));
		frame:AddChild(CreateSpacer(0));
		frame.completeToggleButton = createToggleButton(COMPLETE_ICON,COMPLETE_DISABLED_ICON, KRCLocal:Get("button_tooltip_complete",KeystoneRollCall.MAX_REWARD_LEVEL))
		frame.completeToggleButton:SetPoint("TOPRIGHT",frame.frame,"TOPRIGHT",-14,-116)
		
		frame.altToggleButton = createToggleButton(ALT_ICON,ALT_DISABLED_ICON,KRCLocal:Get("button_tooltip_alts"))
		frame.altToggleButton:SetPoint("TOPRIGHT",frame.completeToggleButton,"TOPLEFT", 1, 0)
		
		frame.offlineToggleButton = createToggleButton(OFFLINE_ICON,OFFLINE_DISABLED_ICON,KRCLocal:Get("button_tooltip_offline"))
		frame.offlineToggleButton:SetPoint("TOPRIGHT",frame.altToggleButton,"TOPLEFT", 1, 0)
		
		frame.keylessToggleButton = createToggleButton(KEYLESS_ICON,KEYLESS_DISABLED_ICON,KRCLocal:Get("button_tooltip_no_keystone"))
		frame.keylessToggleButton:SetPoint("TOPRIGHT",frame.offlineToggleButton,"TOPLEFT", 1, 0)
		
		frame.levelingToggleButton = createToggleButton(LOW_LEVEL_ICON,LOW_LEVEL_DISABLED_ICON,KRCLocal:Get("button_tooltip_leveling"))
		frame.levelingToggleButton:SetPoint("TOPRIGHT",frame.keylessToggleButton,"TOPLEFT", 1, 0)
		
		frame.otherFactionToggleButton = createToggleButton(OPPOSING_FACTION_ICON,OPPOSING_FACTION_DISABLED_ICON,KRCLocal:Get("button_tooltip_other_faction"))
		frame.otherFactionToggleButton:SetPoint("TOPRIGHT",frame.levelingToggleButton,"TOPLEFT", 1, 0)
		
		frame.searchbox = CreateFrame("EditBox", "KRC-SearchBox", frame.frame,"InputBoxTemplate")
		frame.searchbox:SetSize(75, 25)
		--editBox:SetPoint("CENTER")
		frame.searchbox:SetAutoFocus(false)
		frame.searchbox:SetText("")
		frame.searchbox:SetScale(1.55)
		frame.searchbox:SetCursorPosition(0)
		frame.searchbox:SetPoint("BOTTOMRIGHT",frame.otherFactionToggleButton,"BOTTOMLEFT", 0, 0)
		frame.searchbox:SetFontObject(ChatFontNormal)
		frame.searchbox:SetScript("OnEnter", Control_OnEnter)
		frame.searchbox:SetScript("OnLeave", Control_OnLeave)
		frame.searchbox:SetScript("OnEscapePressed", function(box)
			box:ClearFocus()
		end)
		frame.searchbox:SetScript("OnEnterPressed", function(box)
			box:ClearFocus()
		end)
		frame.searchbox:SetScript("OnTextChanged", function(box) 
			KeystoneRollCall:RefreshTab()
		end)
		
		frame.searchbox:SetTextInsets(0, 0, 3, 3)
		frame.searchbox:SetMaxLetters(256)
		frame.searchbox:SetHeight(19)
		frame.searchbox:SetScript("OnEnter", function(box)
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT",box,"TOPLEFT");
			GameTooltip:SetText("Filter Results")
			
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			GameTooltip.bg:Show()
			GameTooltip:Show()
		end)
		frame.searchbox:SetScript("OnLeave", function(box)
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end)
		
		local tabList = {}
		for _,tab in pairs(self.tabs) do
			tinsert(tabList,{text = tab.text, value = tab.value, index = tab.index})
		end
	
		table.sort(tabList,function(a,b)
			return a.index < b.index
		end)

		self.tabFrame =  self.AceGUI:Create("TabGroup")
		self.tabFrame:SetLayout("Flow")
		self.tabFrame:SetTabs(tabList)
		self.tabFrame:SetFullWidth(true);
		self.tabFrame:SetCallback("OnGroupSelected", function(container, event, group)
			container:ReleaseChildren()
			local tab = self.tabs[group]
			self.activeTab = tab.value
			if tab.isKeyTable then
				KeystoneRollCall:SetupDataTable(container,tab.propertyShowAlts)
				container:AddChild(CreateSpacer(0));
				KeystoneRollCall:SetupFilters(tab.showFilterResultsEditBox,  tab.propertyShowComplete, tab.propertyShowAlts, tab.propertyShowOffline, tab.propertyShowKeyless, tab.propertyShowLeveling, tab.propertyShowOpposingFaction)
				KeystoneRollCall:RefreshTab()
			else
				tab.setupContainer(container)
				KeystoneRollCall:HideFilters()
				--container:AddChild(CreateSpacer(0));
			end
		end)
		frame:AddChild(self.tabFrame)
		
		frame.announceGuildButton = CreateFrame("Button", nil, frame.frame,"UIPanelButtonTemplate")
		frame.announceGuildButton:SetSize(100, 30)
		frame.announceGuildButton:SetPoint("BOTTOMLEFT",frame.frame,"BOTTOMLEFT", 13,13)
		
		frame.announceGuildTexture = frame.announceGuildButton:CreateTexture(nil, "ARTWORK")
		frame.announceGuildTexture:SetTexture(ANNOUNCE_ICON)
		frame.announceGuildTexture:SetSize(30, 30)
		frame.announceGuildTexture:SetPoint("BOTTOMLEFT",frame.announceGuildButton,"BOTTOMLEFT", 0,0)
		
		frame.announceGuildLabel = frame.announceGuildButton:CreateFontString()
        frame.announceGuildLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		frame.announceGuildLabel:SetText(KRCLocal:Get("button_to_guild"))
		frame.announceGuildLabel:SetTextColor(1,0.8196079,0,1)
		frame.announceGuildLabel:SetPoint("LEFT",frame.announceGuildTexture,"RIGHT", 0,0)
		
		frame.announceGuildButton:SetScript("OnEnter", function(box)
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT",box,"TOPLEFT");

			GameTooltip:SetText(KRCLocal:Get("button_tooltip_to_guild"))
			GameTooltip:AddLine("|cFFFFFFFF("..KRCLocal:Get("button_tooltip_max",25)..")|r");
			
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			GameTooltip.bg:Show()
			GameTooltip:Show()
		end)
		frame.announceGuildButton:SetScript("OnLeave", function(box)
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end)
		
		frame.announceGuildButton:SetScript("OnClick", function(box)
			KeystoneRollCall:BroadcastListedKeys("GUILD")
		end)
		
		frame.announceGroupButton = CreateFrame("Button", nil, frame.frame,"UIPanelButtonTemplate")
		frame.announceGroupButton:SetSize(100, 30)
		frame.announceGroupButton:SetPoint("LEFT",frame.announceGuildButton,"RIGHT", 4,0)
		
		frame.announceGroupTexture = frame.announceGroupButton:CreateTexture(nil, "ARTWORK")
		frame.announceGroupTexture:SetTexture(ANNOUNCE_ICON)
		frame.announceGroupTexture:SetSize(30, 30)
		frame.announceGroupTexture:SetPoint("BOTTOMLEFT",frame.announceGroupButton,"BOTTOMLEFT", 0,0)
		
		frame.announceGroupLabel = frame.announceGroupButton:CreateFontString()
        frame.announceGroupLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		frame.announceGroupLabel:SetText(KRCLocal:Get("button_to_group"))
		frame.announceGroupLabel:SetTextColor(1,0.8196079,0,1)
		frame.announceGroupLabel:SetPoint("LEFT",frame.announceGroupTexture,"RIGHT", 0,0)
		
		frame.announceGroupButton:SetScript("OnEnter", function(box)
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT",box,"TOPLEFT");

			GameTooltip:SetText(KRCLocal:Get("button_tooltip_to_group"))
			GameTooltip:AddLine("|cFFFFFFFF("..KRCLocal:Get("button_tooltip_max",25)..")|r");
			
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			GameTooltip.bg:Show()
			GameTooltip:Show()
		end)
		frame.announceGroupButton:SetScript("OnLeave", function(box)
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end)
		
		frame.announceGroupButton:SetScript("OnClick", function(box)
			KeystoneRollCall:BroadcastListedKeys(KCLib:GetGroupChatChannel())
		end)
		
		frame.announceTargetButton = CreateFrame("Button", nil, frame.frame,"UIPanelButtonTemplate")
		frame.announceTargetButton:SetSize(100, 30)
		frame.announceTargetButton:SetPoint("LEFT",frame.announceGroupButton,"RIGHT", 4,0)
		
		frame.announceTargetTexture = frame.announceTargetButton:CreateTexture(nil, "ARTWORK")
		frame.announceTargetTexture:SetTexture(ANNOUNCE_ICON)
		frame.announceTargetTexture:SetSize(30, 30)
		frame.announceTargetTexture:SetPoint("BOTTOMLEFT",frame.announceTargetButton,"BOTTOMLEFT", 0,0)
		
		frame.announceTargetLabel = frame.announceTargetButton:CreateFontString()
        frame.announceTargetLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		frame.announceTargetLabel:SetText(KRCLocal:Get("button_to_target"))
		frame.announceTargetLabel:SetTextColor(1,0.8196079,0,1)
		frame.announceTargetLabel:SetPoint("LEFT",frame.announceTargetTexture,"RIGHT", 0,0)
		
		frame.announceTargetButton:SetScript("OnEnter", function(box)
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT",box,"TOPLEFT");

			GameTooltip:SetText(KRCLocal:Get("button_tooltip_to_target"))
			GameTooltip:AddLine("|cFFFFFFFF("..KRCLocal:Get("button_tooltip_max",25)..")|r");
			
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			GameTooltip.bg:Show()
			GameTooltip:Show()
		end)
		frame.announceTargetButton:SetScript("OnLeave", function(box)
		
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end)
		
		frame.announceTargetButton:SetScript("OnClick", function(box)
			if UnitExists("target") and UnitIsPlayer("target") then
				KeystoneRollCall:BroadcastListedKeys("WHISPER", UnitName("target"))
			end
		end)
		
		
		frame.closeButton = CreateFrame("Button", nil, frame.frame,"UIPanelButtonTemplate")
		frame.closeButton:SetSize(100, 30)
		frame.closeButton:SetPoint("BOTTOMRIGHT",frame.frame,"BOTTOMRIGHT", -13,13)
		
		frame.closeButton:SetScript("OnClick", function(self)
			KeystoneRollCall.frame:Hide() 
		end)

		frame.closeLabel = frame.closeButton:CreateFontString()
        frame.closeLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16)
		frame.closeLabel:SetText(KRCLocal:Get("button_close"))
		frame.closeLabel:SetTextColor(1,0.8196079,0,1)
		frame.closeLabel:SetPoint("TOPLEFT",frame.closeButton,"TOPLEFT", 0,0)
		frame.closeLabel:SetPoint("BOTTOMRIGHT",frame.closeButton,"BOTTOMRIGHT", 0,0)
		
		frame.statusBox = CreateFrame("EditBox",nil, frame.frame, "InputBoxTemplate")
		frame.statusBox:SetAutoFocus(false)
		frame.statusBox:SetFont(KeystoneRollCall.db.global.primaryFontPath, 10)
		frame.statusBox:SetFontObject(ChatFontNormal)
		frame.statusBox:SetTextInsets(3, 3, 3, 3)
		frame.statusBox:SetTextColor(1,0.8196079,0,1)
		frame.statusBox:SetMaxLetters(256)
		frame.statusBox:SetPoint("BOTTOMLEFT", 6, 0)
		frame.statusBox:SetPoint("BOTTOMRIGHT")
		frame.statusBox:SetScale(1.2)
		frame.statusBox:SetText(KRCLocal:Get("status_share"))
		frame.statusBox:EnableMouse(false)
		frame.statusBox:SetPoint("TOPLEFT",frame.announceTargetButton,"TOPRIGHT", 10,9)
		frame.statusBox:SetPoint("BOTTOMRIGHT",frame.closeButton,"BOTTOMLEFT", -5,0)
		
		self.tabFrame:SelectTab("personal")
		
		local showSeasonScoreTooltip = function(self)
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			
			local name, _, timeLimit = C_ChallengeMode.GetMapUIInfo(self.dungeonId);
			
			GameTooltip.bg:Show()
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT", self, "TOPLEFT");
			GameTooltip:SetText("|cFFFFFFFF"..name.."|r")
			GameTooltip:AddLine(" ")
	
			local keystone = KCLib:GetPlayerKeystone()
			if keystone and keystone.seasonRunInfos and keystone.seasonRunInfos[self.dungeonId] then
				local level = keystone.seasonRunInfos[self.dungeonId].level or 0;
				local score = keystone.seasonRunInfos[self.dungeonId].score or 0;
				local levelScore = keystone.seasonRunInfos[self.dungeonId].levelScore or 0;
				local completionTime = keystone.seasonRunInfos[self.dungeonId].completionTime or 0;
				local timeScore = keystone.seasonRunInfos[self.dungeonId].timeScore or 0;
				
				
				GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_level")..":", "|cFFFFFFFF"..level.."|r")
				GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_base_score")..":",levelScore)
				GameTooltip:AddLine(" ")
				
				GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_completion_time")..":","|cFFFFFFFF"..KeystoneRollCall:FormatTimeString(completionTime).."|r")
				GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_time_limit")..":","|cFFFFFFFF"..KeystoneRollCall:FormatTimeString(timeLimit).."|r")
				if timeScore > 0 then
					GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_time_bonus")..":",math.floor(timeScore*10)/10)
				elseif timeScore < 0 then
					GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_time_penalty")..":","|cFFFF3333"..(math.floor(timeScore*10)/10).."|r")
				end
				GameTooltip:AddLine(" ")
				GameTooltip:AddDoubleLine(KRCLocal:Get("tooltip_score")..":",math.floor(score*10)/10)
			else
				GameTooltip:AddLine(KRCLocal:Get("tooltip_no_run_found_season"))
			end
	
	
				
			
			--GameTooltip:AddLine(desc)
			GameTooltip:Show()
		end
		
		local showAffix = function(self)
			local name,desc = C_ChallengeMode.GetAffixInfo(C_MythicPlus.GetCurrentAffixes()[self.affix].id);
			
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			GameTooltip.bg:Show()
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT", self, "TOPLEFT");
			GameTooltip:SetText("|cFFFFFFFF"..name.."|r")
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine(desc)
			GameTooltip:Show()
		end
		
		local hideAffix = function(box)
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end
		
		
		local lastBtn;
		local width = (721 - #KeystoneRollCall.sortedMaps * 3) / #KeystoneRollCall.sortedMaps
		frame.seasonBestButtons = {}
		local keystone = KCLib:GetPlayerKeystone()
	
		for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
			local btn = CreateFrame("Button", nil, frame.frame)
			btn.dungeonId = dungeonId;
			btn:SetSize(width, 40.5)
			
			local txt = btn:CreateTexture(nil, "BACKGROUND")
			txt:SetPoint("TOPLEFT",btn,"TOPLEFT",0,.25)
			txt:SetWidth(width)
			txt:SetHeight(40.5)
			txt:SetTexture(select(4,C_ChallengeMode.GetMapUIInfo(dungeonId)))
			
			if lastBtn then
				local div1 = frame.frame:CreateTexture(nil)
				div1:SetColorTexture(.30,.30,.30,1)
				div1:SetPoint("TOP",lastBtn,"TOP",0,0)
				div1:SetPoint("BOTTOM",lastBtn,"BOTTOM",0,0)
				div1:SetPoint("LEFT",lastBtn,"RIGHT",1,0)
				div1:SetWidth(1)
				div1:SetHeight(40.5)
				local div2 = frame.frame:CreateTexture(nil)
				div2:SetColorTexture(.45,.45,.45,1)
				div2:SetPoint("TOP",div1,"TOP",0,0)
				div2:SetPoint("BOTTOM",div1,"BOTTOM",0,0)
				div2:SetPoint("LEFT",div1,"RIGHT",1,0)
				div2:SetWidth(1)
				div2:SetHeight(40.5)
				local div3 = frame.frame:CreateTexture(nil)
				div3:SetColorTexture(.30,.30,.30,1)
				div3:SetPoint("TOP",div2,"TOP",0,0)
				div3:SetPoint("BOTTOM",div2,"BOTTOM",0,0)
				div3:SetPoint("LEFT",div2,"RIGHT",1,0)
				div3:SetWidth(1)
				div3:SetHeight(40.5)
			
				btn:SetPoint("LEFT",div3,"RIGHT", 0, 0)
			else
				btn:SetPoint("TOPLEFT",frame.scoreBG,"TOPLEFT", 1, 0)
			end
			btn.affix = 1
			btn:SetScript("OnEnter", showSeasonScoreTooltip)
			btn:SetScript("OnLeave", hideAffix)
			frame.seasonBestButtons[dungeonId] = btn;
				
			local levelLabel = btn:CreateFontString()
			levelLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 19, "THICKOUTLINE")
			levelLabel:SetJustifyH("CENTER")
			levelLabel:SetTextColor(1,0.8196079,0,1)
			levelLabel:SetPoint("TOPLEFT",btn,"TOPLEFT",0,0)
			levelLabel:SetPoint("TOPRIGHT",btn,"TOPRIGHT",0,0)
			levelLabel:SetHeight(25)
			
			btn.levelLabel = levelLabel;
				
							
			local scoreValueLabel = btn:CreateFontString()
			scoreValueLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "THICKOUTLINE")
			scoreValueLabel:SetJustifyH("CENTER")
			scoreValueLabel:SetTextColor(1,0.8196079,0,1)
			scoreValueLabel:SetPoint("BOTTOMLEFT",btn,"BOTTOMLEFT",0,0)
			scoreValueLabel:SetPoint("BOTTOMRIGHT",btn,"BOTTOMRIGHT",0,0)
			scoreValueLabel:SetHeight(15)
			btn.scoreValueLabel = scoreValueLabel;
			
			
			
			lastBtn = btn;
		end
		
		
		frame.affix1Button = CreateFrame("Button", nil, frame.frame)
		frame.affix1Button:SetSize(175, 30)
		frame.affix1Button:SetPoint("TOPLEFT",frame.affixBG,"TOPLEFT", 5, -6)
		frame.affix1Button.affix = 1
		frame.affix1Button:SetScript("OnEnter", showAffix)
		frame.affix1Button:SetScript("OnLeave", hideAffix)
		
		frame.affix1Icon = frame.affix1Button:CreateTexture()
		frame.affix1Icon:SetPoint("TOPLEFT",frame.affix1Button,"TOPLEFT",0,0)
		frame.affix1Icon:SetWidth(30)
		frame.affix1Icon:SetHeight(30)
	
		frame.affix1Label = frame.affix1Button:CreateFontString()
        frame.affix1Label:SetFont(KeystoneRollCall.db.global.primaryFontPath, 22)
		frame.affix1Label:SetText("Affix 1")
		frame.affix1Label:SetJustifyH("LEFT")
		frame.affix1Label:SetTextColor(1,0.8196079,0,1)
		frame.affix1Label:SetPoint("TOPLEFT",frame.affix1Icon,"TOPRIGHT",5,0)
		frame.affix1Label:SetPoint("BOTTOMRIGHT",frame.affix1Button,"BOTTOMRIGHT",5,0)
		
		frame.affix2Button = CreateFrame("Button", nil, frame.frame)
		frame.affix2Button:SetSize(175, 30)
		frame.affix2Button:SetPoint("LEFT",frame.affix1Button,"RIGHT", 5, 0)
		frame.affix2Button.affix = 2
		frame.affix2Button:SetScript("OnEnter", showAffix)
		frame.affix2Button:SetScript("OnLeave", hideAffix)
		
		frame.affix2Icon = frame.affix2Button:CreateTexture()
		frame.affix2Icon:SetPoint("TOPLEFT",frame.affix2Button,"TOPLEFT",0,0)
		frame.affix2Icon:SetWidth(30)
		frame.affix2Icon:SetHeight(30)
	
		frame.affix2Label = frame.affix2Button:CreateFontString()
        frame.affix2Label:SetFont(KeystoneRollCall.db.global.primaryFontPath, 22)
		frame.affix2Label:SetText("Affix 1")
		frame.affix2Label:SetJustifyH("LEFT")
		frame.affix2Label:SetTextColor(1,0.8196079,0,1)
		frame.affix2Label:SetPoint("TOPLEFT",frame.affix2Icon,"TOPRIGHT",5,0)
		frame.affix2Label:SetPoint("BOTTOMRIGHT",frame.affix2Button,"BOTTOMRIGHT",5,0)
		
		frame.affix3Button = CreateFrame("Button", nil, frame.frame)
		frame.affix3Button:SetSize(175, 30)
		frame.affix3Button:SetPoint("LEFT",frame.affix2Button,"RIGHT", 5, 0)
		frame.affix3Button.affix = 3
		frame.affix3Button:SetScript("OnEnter", showAffix)
		frame.affix3Button:SetScript("OnLeave", hideAffix)
		
		frame.affix3Icon = frame.affix3Button:CreateTexture()
		frame.affix3Icon:SetPoint("TOPLEFT",frame.affix3Button,"TOPLEFT",0,0)
		frame.affix3Icon:SetWidth(30)
		frame.affix3Icon:SetHeight(30)
	
		frame.affix3Label = frame.affix3Button:CreateFontString()
        frame.affix3Label:SetFont(KeystoneRollCall.db.global.primaryFontPath, 22)
		frame.affix3Label:SetText("Affix 1")
		frame.affix3Label:SetJustifyH("LEFT")
		frame.affix3Label:SetTextColor(1,0.8196079,0,1)
		frame.affix3Label:SetPoint("TOPLEFT",frame.affix3Icon,"TOPRIGHT",5,0)
		frame.affix3Label:SetPoint("BOTTOMRIGHT",frame.affix3Button,"BOTTOMRIGHT",5,0)
		
		frame.showAffixScheduleButton = CreateFrame("Button", nil, frame.frame,"UIPanelButtonTemplate")
		frame.showAffixScheduleButton:SetSize(175, 30)
		frame.showAffixScheduleButton:SetPoint("LEFT",frame.affix3Button,"RIGHT", 5,0)
		frame.showAffixScheduleButton:SetPoint("RIGHT",frame.affixBG,"RIGHT",-5,0)
		
		frame.showAffixScheduleIcon = frame.showAffixScheduleButton:CreateTexture()
		frame.showAffixScheduleIcon:SetPoint("TOPLEFT",frame.showAffixScheduleButton,"TOPLEFT",0,0)
		frame.showAffixScheduleIcon:SetTexture(SCHEDULE_ICON)
		frame.showAffixScheduleIcon:SetWidth(30)
		frame.showAffixScheduleIcon:SetHeight(30)
	
		frame.showAffixScheduleLabel = frame.showAffixScheduleButton:CreateFontString()
        frame.showAffixScheduleLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
		frame.showAffixScheduleLabel:SetText(KRCLocal:Get("button_affix_schedule"))
		frame.showAffixScheduleLabel:SetJustifyH("LEFT")
		frame.showAffixScheduleLabel:SetTextColor(1,0.8196079,0,1)
		frame.showAffixScheduleLabel:SetPoint("TOPLEFT",frame.showAffixScheduleIcon,"TOPRIGHT",5,0)
		frame.showAffixScheduleLabel:SetPoint("BOTTOMRIGHT",frame.showAffixScheduleButton,"BOTTOMRIGHT",5,0)
		
		frame.showAffixScheduleButton:SetScript("OnEnter", function(self)
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:ClearAllPoints();
			GameTooltip:SetPoint("BOTTOMLEFT", self, "TOPLEFT");

			GameTooltip:SetText(KRCLocal:Get("button_tooltip_affix_schedule1"))
			GameTooltip:AddLine("|cFFFFFFFF("..KRCLocal:Get("button_tooltip_affix_schedule2")..")|r");
			
			
			if not GameTooltip.bg then
				GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
				GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
				GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
				GameTooltip.bg:SetColorTexture(0,0,0,1)

			end
			GameTooltip.bg:Show()
			GameTooltip:Show()
		end)
		frame.showAffixScheduleButton:SetScript("OnLeave", function(self)
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end)
		frame.showAffixScheduleButton:SetScript("OnClick", function(self)
			KeystoneRollCall:ShowScheduleFrame(function() 
				KeystoneRollCall.frame:Show()
				KeystoneRollCall:RefreshTab()
			end)
			
			KeystoneRollCall.frame:Hide()
		end)
		
		KeystoneRollCall:AddSettingsListener(ApplySettings);
		ApplySettings();
	end
	UpdateSeasonBestDisplay();
	local affixes = C_MythicPlus.GetCurrentAffixes();
	self.frame.affix1Icon:SetTexture(select(3,C_ChallengeMode.GetAffixInfo(affixes[1].id)))
	self.frame.affix2Icon:SetTexture(select(3,C_ChallengeMode.GetAffixInfo(affixes[2].id)))
	self.frame.affix3Icon:SetTexture(select(3,C_ChallengeMode.GetAffixInfo(affixes[3].id)))
	self.frame.affix1Label:SetText(C_ChallengeMode.GetAffixInfo(affixes[1].id))
	self.frame.affix2Label:SetText(C_ChallengeMode.GetAffixInfo(affixes[2].id))
	self.frame.affix3Label:SetText(C_ChallengeMode.GetAffixInfo(affixes[3].id))
end

function KeystoneRollCall:SetStatus(text)
	if not text or string.len(text) == 0 then
		if KeystoneRollCall.filtered > 0 then
			self.frame.statusBox:SetText(KRCLocal:Get("status_filtered",#KeystoneRollCall.tableData, KeystoneRollCall.filtered));
		else
			self.frame.statusBox:SetText(KRCLocal:Get("status_listed",#KeystoneRollCall.tableData));
		end
	else
		self.frame.statusBox:SetText(text)
	end
end

KeystoneRollCall.listingOnClick = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
	if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and KeystoneRollCall.tableData[realrow][1] then
		local keystone = KeystoneRollCall.tableData[realrow][1];
		
		if keystone.dummy then
			KeystoneRollCall:SuggestAddon("WHISPER",keystone.name)
		else
			KeystoneRollCall.frame:Hide();
			KeystoneRollCall:ShowShareFrame(keystone,function() 
				KeystoneRollCall.frame:Show();
			end)
		end
	end
end

KeystoneRollCall.listingOnLeave = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
	KeystoneRollCall:SetStatus("")
	if GameTooltip.keystoneElements then
		for key,element in pairs(GameTooltip.keystoneElements) do
			if element.Hide then
				element:Hide()
			end
		end
	end

	
	if GameTooltip.bg then
		GameTooltip.bg:Hide()
	end
	GameTooltip:Hide()
end
		
KeystoneRollCall.listingOnEntry = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
	if not realrow or not KeystoneRollCall.tableData[realrow] then
		return
	end

	local key = KeystoneRollCall.tableData[realrow][1]
	local data = KeystoneRollCall:GetCharacterData(key.name)
	if key.dummy then
		KeystoneRollCall:SetStatus(KRCLocal:Get("status_suggest"))
	else
		KeystoneRollCall:SetStatus(KRCLocal:Get("status_share"))
	end
	
	if not GameTooltip.bg then
		GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
		GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
		GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
		GameTooltip.bg:SetColorTexture(0,0,0,1)
	end

	
	if data then
		GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
		GameTooltip:SetText(" ")
		
		if not GameTooltip.keystoneElements then
			GameTooltip.keystoneElements = {};
			
			

			
			GameTooltip.keystoneElements.characterName = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.characterName:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16)
			GameTooltip.keystoneElements.characterName:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.characterName:SetPoint("TOP",GameTooltip,"TOP",0,-8)
			
			GameTooltip.keystoneElements.guildName = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.guildName:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
			GameTooltip.keystoneElements.guildName:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.guildName:SetPoint("TOP",GameTooltip.keystoneElements.characterName,"BOTTOM",0,-2)

			GameTooltip.keystoneElements.characterDesc = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.characterDesc:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
			GameTooltip.keystoneElements.characterDesc:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.characterDesc:SetPoint("TOP",GameTooltip.keystoneElements.guildName,"BOTTOM",0,-2)

			GameTooltip.keystoneElements.ilvlLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.ilvlLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
			GameTooltip.keystoneElements.ilvlLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.ilvlLine:SetPoint("TOP",GameTooltip.keystoneElements.characterDesc,"BOTTOM",0,-2)
			
			GameTooltip.keystoneElements.classIcon = GameTooltip:CreateTexture(nil,"CENTER"); -- create texture as child of myframe
			GameTooltip.keystoneElements.classIcon:SetPoint("RIGHT",GameTooltip.keystoneElements.characterName,"LEFT",-3,1);
			GameTooltip.keystoneElements.classIcon:SetSize(20,20)
			GameTooltip.keystoneElements.classIcon:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES"); -- this is the image containing all class icons			
				
			GameTooltip.keystoneElements.divider1 = GameTooltip:CreateTexture(nil)
			GameTooltip.keystoneElements.divider1:SetColorTexture(.20,.20,.20,1)
			GameTooltip.keystoneElements.divider1:SetPoint("TOP",GameTooltip.keystoneElements.ilvlLine,"BOTTOM",0,-3)
			GameTooltip.keystoneElements.divider1:SetPoint("LEFT",GameTooltip,"LEFT",3,0)
			GameTooltip.keystoneElements.divider1:SetPoint("RIGHT",GameTooltip,"RIGHT",-5,0)
			GameTooltip.keystoneElements.divider1:SetHeight(2)
			
			
			
			GameTooltip.keystoneElements.seasonScoreLabelLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.seasonScoreLabelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 14)
			GameTooltip.keystoneElements.seasonScoreLabelLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.seasonScoreLabelLine:SetTextColor(1,0.8196079,0,1)
			GameTooltip.keystoneElements.seasonScoreLabelLine:SetText(KRCLocal:Get("tooltip_season_score"))
			GameTooltip.keystoneElements.seasonScoreLabelLine:SetPoint("TOP",GameTooltip.keystoneElements.divider1,"BOTTOM",0,-8)
			
			GameTooltip.keystoneElements.seasonScoreLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.seasonScoreLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 20)
			GameTooltip.keystoneElements.seasonScoreLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.seasonScoreLine:SetPoint("TOP",GameTooltip.keystoneElements.seasonScoreLabelLine,"BOTTOM",0,-8)
			
			GameTooltip.keystoneElements.divider2 = GameTooltip:CreateTexture(nil)
			GameTooltip.keystoneElements.divider2:SetColorTexture(.20,.20,.20,1)
			GameTooltip.keystoneElements.divider2:SetPoint("TOP",GameTooltip.keystoneElements.seasonScoreLine,"BOTTOM",0,-1)
			GameTooltip.keystoneElements.divider2:SetPoint("LEFT",GameTooltip,"LEFT",3,0)
			GameTooltip.keystoneElements.divider2:SetPoint("RIGHT",GameTooltip,"RIGHT",-5,0)
			GameTooltip.keystoneElements.divider2:SetHeight(1)
			
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 14)
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetTextColor(1,0.8196079,0,1)
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetText(KRCLocal:Get("tooltip_account_season_score"))	
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetPoint("LEFT",GameTooltip,"LEFT",3,0)
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetPoint("RIGHT",GameTooltip,"RIGHT",-5,0)
			
			GameTooltip.keystoneElements.accountSeasonScoreLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.accountSeasonScoreLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 20)
			GameTooltip.keystoneElements.accountSeasonScoreLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.accountSeasonScoreLine:SetPoint("TOP",GameTooltip.keystoneElements.accountSeasonScoreLabelLine,"BOTTOM",0,-8)
			
			GameTooltip.keystoneElements.divider3 = GameTooltip:CreateTexture(nil)
			GameTooltip.keystoneElements.divider3:SetColorTexture(.20,.20,.20,1)
			GameTooltip.keystoneElements.divider3:SetPoint("TOP",GameTooltip.keystoneElements.accountSeasonScoreLine,"BOTTOM",0,-1)
			GameTooltip.keystoneElements.divider3:SetPoint("LEFT",GameTooltip,"LEFT",3,0)
			GameTooltip.keystoneElements.divider3:SetPoint("RIGHT",GameTooltip,"RIGHT",-5,0)
			GameTooltip.keystoneElements.divider3:SetHeight(1)
				
			for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
				local name,_,_,texturePath = C_ChallengeMode.GetMapUIInfo(dungeonId);
				
				local texture = GameTooltip:CreateTexture(nil)
				texture:SetTexture(texturePath)
				texture:SetSize(14,14)
				texture:SetPoint("LEFT",GameTooltip,"LEFT",8,0)
				
				local accountTexture = GameTooltip:CreateTexture(nil)
				accountTexture:SetTexture(texturePath)
				accountTexture:SetSize(14,14)
				accountTexture:SetPoint("LEFT",GameTooltip,"LEFT",8,0)
				
				if i == 1 then
					texture:SetPoint("TOP",GameTooltip.keystoneElements.seasonScoreLine,"BOTTOM",0,-6)
					accountTexture:SetPoint("TOP",GameTooltip.keystoneElements.accountSeasonScoreLine,"BOTTOM",0,-6)
				else
					texture:SetPoint("TOP",GameTooltip.keystoneElements["map"..(i-1).."texture"],"BOTTOM",0,-3)
					accountTexture:SetPoint("TOP",GameTooltip.keystoneElements["map"..(i-1).."accounttexture"],"BOTTOM",0,-3)
				end
				GameTooltip.keystoneElements["map"..i.."texture"] = texture;
				GameTooltip.keystoneElements["map"..i.."accounttexture"] = accountTexture;
				
				local nameLine = GameTooltip:CreateFontString()
				nameLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
				nameLine:SetTextColor(1,0.8196079,0,1)
				nameLine:SetText(name)
				nameLine:SetPoint("Left",texture,"Right",5,0)
				GameTooltip.keystoneElements["map"..i.."name"] = nameLine;
				
				local accountNameLine = GameTooltip:CreateFontString()
				accountNameLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
				accountNameLine:SetTextColor(1,0.8196079,0,1)
				accountNameLine:SetText(name)
				accountNameLine:SetPoint("Left",accountTexture,"Right",5,0)
				GameTooltip.keystoneElements["map"..i.."accountname"] = accountNameLine;
				
				local levelLine = GameTooltip:CreateFontString()
				levelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
				levelLine:SetTextColor(1,0.8196079,0,1)
				levelLine:SetPoint("TOP",nameLine,"TOP",5,0)
				levelLine:SetPoint("RIGHT",GameTooltip,"RIGHT",-18,0)
				GameTooltip.keystoneElements["map"..i.."level"] = levelLine;
				
				local accountLevelLine = GameTooltip:CreateFontString()
				accountLevelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 13)
				accountLevelLine:SetTextColor(1,0.8196079,0,1)
				accountLevelLine:SetPoint("TOP",accountNameLine,"TOP",5,0)
				accountLevelLine:SetPoint("RIGHT",GameTooltip,"RIGHT",-18,0)
				GameTooltip.keystoneElements["map"..i.."accountlevel"] = accountLevelLine;
				
				local divider = GameTooltip:CreateTexture(nil)
				divider:SetColorTexture(.20,.20,.20,1)
				divider:SetPoint("TOP",texture,"BOTTOM",0,0)
				divider:SetPoint("LEFT",GameTooltip,"LEFT",3,0)
				divider:SetPoint("RIGHT",GameTooltip,"RIGHT",-5,0)
				divider:SetHeight(1)
				GameTooltip.keystoneElements["map"..i.."divider"] = divider;
				
				local accountDivider = GameTooltip:CreateTexture(nil)
				accountDivider:SetColorTexture(.20,.20,.20,1)
				accountDivider:SetPoint("TOP",accountTexture,"BOTTOM",0,0)
				accountDivider:SetPoint("LEFT",GameTooltip,"LEFT",3,0)
				accountDivider:SetPoint("RIGHT",GameTooltip,"RIGHT",-5,0)
				accountDivider:SetHeight(1)
				GameTooltip.keystoneElements["map"..i.."accountdivider"] = accountDivider;
				
				
				local star1 = GameTooltip:CreateTexture(nil)
				star1:SetTexture(GOLD_STAR_ICON)
				star1:SetTexCoord(0,1,0,1)
				star1:SetPoint("TOPRIGHT",levelLine,"TOPLEFT",-2,0)
				star1:SetSize(5,5)
				GameTooltip.keystoneElements["map"..i.."star1"] = star1;
				
				local star2 = GameTooltip:CreateTexture(nil)
				star2:SetTexture(GOLD_STAR_ICON)
				star2:SetTexCoord(0,1,0,1)
				star2:SetPoint("TOPRIGHT",star1,"TOPLEFT",-2,0)
				star2:SetSize(5,5)
				GameTooltip.keystoneElements["map"..i.."star2"] = star2;
				
				local star3 = GameTooltip:CreateTexture(nil)
				star3:SetTexture(GOLD_STAR_ICON)
				star3:SetPoint("TOPRIGHT",star2,"TOPLEFT",-2,0)
				star3:SetSize(5,5)
				GameTooltip.keystoneElements["map"..i.."star3"] = star3;
				
				local accountStar1 = GameTooltip:CreateTexture(nil)
				accountStar1:SetTexture(GOLD_STAR_ICON)
				accountStar1:SetPoint("TOPRIGHT",accountLevelLine,"TOPLEFT",-2,0)
				accountStar1:SetSize(5,5)
				GameTooltip.keystoneElements["map"..i.."accountstar1"] = accountStar1;
				
				local accountStar2 = GameTooltip:CreateTexture(nil)
				accountStar2:SetTexture(GOLD_STAR_ICON)
				accountStar2:SetPoint("TOPRIGHT",accountStar1,"TOPLEFT",-2,0)
				accountStar2:SetSize(5,5)
				GameTooltip.keystoneElements["map"..i.."accountstar2"] = accountStar2;
				
				local accountStar3 = GameTooltip:CreateTexture(nil)
				accountStar3:SetTexture(GOLD_STAR_ICON)
				accountStar3:SetPoint("TOPRIGHT",accountStar2,"TOPLEFT",-2,0)
				accountStar3:SetSize(5,5)
				GameTooltip.keystoneElements["map"..i.."accountstar3"] = accountStar3;
			end
			
			
			
			
			GameTooltip.keystoneElements.knownAltsLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.knownAltsLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
			GameTooltip.keystoneElements.knownAltsLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.knownAltsLine:SetPoint("BOTTOM",GameTooltip,"BOTTOM",0,25)
			GameTooltip.keystoneElements.knownAltsLine:SetPoint("LEFT",GameTooltip,"LEFT",6,0)
			GameTooltip.keystoneElements.knownAltsLine:SetPoint("RIGHT",GameTooltip,"RIGHT",-8,0)
			
			GameTooltip.keystoneElements.knownAltsLabelLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.knownAltsLabelLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 14)
			GameTooltip.keystoneElements.knownAltsLabelLine:SetJustifyH("CENTER")
			GameTooltip.keystoneElements.knownAltsLabelLine:SetTextColor(1,0.8196079,0,1)
			GameTooltip.keystoneElements.knownAltsLabelLine:SetText(KRCLocal:Get("tooltip_known_alts"))
			GameTooltip.keystoneElements.knownAltsLabelLine:SetPoint("BOTTOM",GameTooltip.keystoneElements.knownAltsLine,"TOP",0,5)

			
			GameTooltip.keystoneElements.lastSeenLine = GameTooltip:CreateFontString()
			GameTooltip.keystoneElements.lastSeenLine:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12)
			GameTooltip.keystoneElements.lastSeenLine:SetJustifyH("RIGHT")
			GameTooltip.keystoneElements.lastSeenLine:SetPoint("BOTTOMRIGHT",GameTooltip,"BOTTOMRIGHT",-7,5)
			
		end
		local classIconCoords = CLASS_ICON_TCOORDS[data.class or key.class];
		if classIconCoords then
			GameTooltip.keystoneElements.classIcon:SetTexCoord(unpack(classIconCoords)); 
			GameTooltip.keystoneElements.classIcon:Show()
		else
			GameTooltip.keystoneElements.classIcon:Hide()
		end
		GameTooltip.keystoneElements.characterName:SetText(KeystoneRollCall:colorClass(data.name or "",data.class or ""))
		GameTooltip.keystoneElements.characterName:Show()
		if key.guildName and string.len(key.guildName) > 0 then
			GameTooltip.keystoneElements.guildName:SetText("<"..key.guildName..">")
			GameTooltip.keystoneElements.characterDesc:SetPoint("TOP",GameTooltip.keystoneElements.guildName,"BOTTOM",0,-2)
			GameTooltip.keystoneElements.guildName:Show()
		else
			GameTooltip.keystoneElements.guildName:SetText("")
			GameTooltip.keystoneElements.characterDesc:SetPoint("TOP",GameTooltip.keystoneElements.characterName,"BOTTOM",0,-2)
		end
		
		local descString = ""
		if key.guid then 
			descString = KeystoneRollCall:colorFaction(select(3,GetPlayerInfoByGUID("Player-"..key.guid)),key.faction)
		else
			descString = KeystoneRollCall:colorFaction(key.faction and "Alliance" or "Horde",key.faction)
		end
		
		if data.class then 
			descString = descString .. " - ".. KeystoneRollCall:colorClass(KRCLocal:Get(data.class),data.class)
		end
		
		if key.level then
			descString = descString .. " - ".. key.level
		end
		
		GameTooltip.keystoneElements.characterDesc:SetText(descString)
		GameTooltip.keystoneElements.characterDesc:Show()
		if key.ilvl then
			GameTooltip.keystoneElements.ilvlLine:SetText(KRCLocal:Get("tooltip_item_level") .. " " .. (key.ilvl or KRCLocal:Get("error_unknown"))  .. " (" .. KRCLocal:Get("tooltip_equipped") .. " " .. (key.ilvlEquipped  or KRCLocal:Get("error_unknown")) ..")")
			GameTooltip.keystoneElements.ilvlLine:Show()
		else
			GameTooltip.keystoneElements.ilvlLine:Hide()
		end
		GameTooltip.keystoneElements.divider1:Show()		
		
		GameTooltip.keystoneElements.seasonScoreLabelLine:Show()
		local bestSeasonScore = key.seasonScore or 0
		GameTooltip.keystoneElements.seasonScoreLine:SetText(bestSeasonScore)
		GameTooltip.keystoneElements.seasonScoreLine:Show()
		GameTooltip.keystoneElements.divider2:Show()
		
		local accountBestRunInfo = {};
		local lastMapTexture = nil
		for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
			if key.seasonRunInfos and key.seasonRunInfos[dungeonId] and key.seasonRunInfos[dungeonId].level > 0 then
				local runInfo = key.seasonRunInfos[dungeonId];
				accountBestRunInfo[dungeonId] = runInfo
				local texture = GameTooltip.keystoneElements["map"..i.."texture"];
				if not lastMapTexture then
					texture:SetPoint("TOP",GameTooltip.keystoneElements.seasonScoreLine,"BOTTOM",0,-4)
				else
					texture:SetPoint("TOP",lastMapTexture,"BOTTOM",0,-1)
				end
				texture:Show();
				local upgradeCount = runInfo.keystoneUpgradeLevels or 0;
				if upgradeCount > 0 then
					GameTooltip.keystoneElements["map"..i.."star1"]:Show()
					if upgradeCount > 1 then
						GameTooltip.keystoneElements["map"..i.."star2"]:Show()
						if upgradeCount > 2 then
							GameTooltip.keystoneElements["map"..i.."star3"]:Show()
						else
							GameTooltip.keystoneElements["map"..i.."star3"]:Hide()
						end
					else
						GameTooltip.keystoneElements["map"..i.."star2"]:Hide()
						GameTooltip.keystoneElements["map"..i.."star3"]:Hide()
					end
				else
					GameTooltip.keystoneElements["map"..i.."star1"]:Hide()
					GameTooltip.keystoneElements["map"..i.."star2"]:Hide()
					GameTooltip.keystoneElements["map"..i.."star3"]:Hide()
				end
				
				GameTooltip.keystoneElements["map"..i.."name"]:Show()
				GameTooltip.keystoneElements["map"..i.."level"]:SetText(runInfo.level)
				GameTooltip.keystoneElements["map"..i.."level"]:Show()
				GameTooltip.keystoneElements["map"..i.."divider"]:Show()
				lastMapTexture = texture;
			else
				accountBestRunInfo[dungeonId] = {["level"]=0,["score"]=0}
				GameTooltip.keystoneElements["map"..i.."texture"]:Hide();
				GameTooltip.keystoneElements["map"..i.."name"]:Hide()
				GameTooltip.keystoneElements["map"..i.."level"]:Hide()
				GameTooltip.keystoneElements["map"..i.."divider"]:Hide()
			end
		end
		
		local count = 0
		local altString = ""
		KeystoneRollCall:ForEachAltKeystoneEntry(key.name,function(altKeystone,mainsKeystone)
			if altKeystone.name then
				for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
					if altKeystone.seasonRunInfos and altKeystone.seasonRunInfos[dungeonId] and altKeystone.seasonRunInfos[dungeonId].score and altKeystone.seasonRunInfos[dungeonId].score > accountBestRunInfo[dungeonId].score then
						accountBestRunInfo[dungeonId] = altKeystone.seasonRunInfos[dungeonId]
					end
				end
				if count < 12 then
					if count ~= 0 then
						altString = altString ..", "
					end
					local altNameParts =  {strsplit("-",altKeystone.name)};
					altString = altString .. KeystoneRollCall:colorClass(altNameParts[1],altKeystone.class)
				end
				count = count + 1
			end
		end)
		
		if count > 12 then
			altString = altString .. ", "..KRCLocal:Get("broadcast_and_more",count-12)
		end
		
		if not lastMapTexture then
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetPoint("TOP",GameTooltip.keystoneElements.seasonScoreLine,"BOTTOM",0,-18)
		else
			GameTooltip.keystoneElements.accountSeasonScoreLabelLine:SetPoint("TOP",lastMapTexture,"BOTTOM",0,-15)
		end
		
		GameTooltip.keystoneElements.accountSeasonScoreLabelLine:Show()
		GameTooltip.keystoneElements.accountSeasonScoreLine:Show()
		GameTooltip.keystoneElements.divider3:Show()
		
		local accountWideScore = 0;
		lastMapTexture = nil
		for i, dungeonId in ipairs(KeystoneRollCall.sortedMaps) do
			if accountBestRunInfo[dungeonId].level > 0 then
				accountWideScore = accountWideScore + (accountBestRunInfo[dungeonId].score or 0)
				local texture = GameTooltip.keystoneElements["map"..i.."accounttexture"];
				if not lastMapTexture then
					texture:SetPoint("TOP",GameTooltip.keystoneElements.accountSeasonScoreLine,"BOTTOM",0,-4)
				else
					texture:SetPoint("TOP",lastMapTexture,"BOTTOM",0,-1)
				end
				texture:Show();
				local upgradeCount = accountBestRunInfo[dungeonId].keystoneUpgradeLevels or 0;
				if upgradeCount > 0 then
					GameTooltip.keystoneElements["map"..i.."accountstar1"]:Show()
					if upgradeCount > 1 then
						GameTooltip.keystoneElements["map"..i.."accountstar2"]:Show()
						if upgradeCount > 2 then
							GameTooltip.keystoneElements["map"..i.."accountstar3"]:Show()
						else
							GameTooltip.keystoneElements["map"..i.."accountstar3"]:Hide()
						end
					else
						GameTooltip.keystoneElements["map"..i.."accountstar2"]:Hide()
						GameTooltip.keystoneElements["map"..i.."accountstar3"]:Hide()
					end
				else
					GameTooltip.keystoneElements["map"..i.."accountstar1"]:Hide()
					GameTooltip.keystoneElements["map"..i.."accountstar2"]:Hide()
					GameTooltip.keystoneElements["map"..i.."accountstar3"]:Hide()
				end
				GameTooltip.keystoneElements["map"..i.."accountname"]:Show()
				GameTooltip.keystoneElements["map"..i.."accountlevel"]:SetText(accountBestRunInfo[dungeonId].level)
				GameTooltip.keystoneElements["map"..i.."accountlevel"]:Show()
				GameTooltip.keystoneElements["map"..i.."accountdivider"]:Show()
				lastMapTexture = texture;
			else
				GameTooltip.keystoneElements["map"..i.."accounttexture"]:Hide();
				GameTooltip.keystoneElements["map"..i.."accountname"]:Hide()
				GameTooltip.keystoneElements["map"..i.."accountlevel"]:Hide()
				GameTooltip.keystoneElements["map"..i.."accountdivider"]:Hide()
				GameTooltip.keystoneElements["map"..i.."accountstar1"]:Hide()
				GameTooltip.keystoneElements["map"..i.."accountstar2"]:Hide()
				GameTooltip.keystoneElements["map"..i.."accountstar3"]:Hide()
			end
		end
		GameTooltip.keystoneElements.accountSeasonScoreLine:SetText(math.floor(accountWideScore))
		
		if count > 0 then		
			GameTooltip.keystoneElements.knownAltsLabelLine:Show()
			GameTooltip.keystoneElements.knownAltsLine:SetText(altString)
			GameTooltip.keystoneElements.knownAltsLine:Show()
		else
			GameTooltip.keystoneElements.knownAltsLabelLine:Hide()
			GameTooltip.keystoneElements.knownAltsLine:Hide()
		end
		
		GameTooltip.keystoneElements.lastSeenLine:SetText(date("%m/%d/%y",data.lastSeen))
		GameTooltip.keystoneElements.lastSeenLine:Show()
		GameTooltip.bg:Show()
		GameTooltip:Show()
		GameTooltip:ClearAllPoints();
		GameTooltip:SetPoint("TOPLEFT",KeystoneRollCall.frame.frame,"TOPRIGHT",-5,-2);
		GameTooltip:SetPoint("TOPRIGHT",KeystoneRollCall.frame.frame,"TOPRIGHT",297,0);
		GameTooltip:SetPoint("BOTTOMLEFT",KeystoneRollCall.frame.frame,"BOTTOMRIGHT",0,5);
	end
end

local function ApplySettingsToCell(rowFrame, cellFrame, data, cols, row, realrow, column, fShow, table, ...)
	cellFrame.text:SetText(data[realrow][column])
	cellFrame.text:SetFont(KeystoneRollCall.db.global.primaryFontPath,10)
end

function KeystoneRollCall:SetupDataTable(container,supportsAltDisplay)	
	local cols = {{
			['name'] = '',
			['width'] = 1,
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "name",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_realm"),
			['width'] = 85,
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "realm",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_character"),
			['width'] = 85,
			['align'] = 'LEFT', 
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "name",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_class"),
			['width'] = 90,
			['align'] = 'LEFT',
			['field'] = "class", 
			["comparesort"] = KeystoneRollCall.columnSort,
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_ilvl"),
			['width'] = 50,
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "ilvl",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_score"),
			['width'] = 40,
			['align'] = 'RIGHT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "seasonScore",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_level"),
			['width'] = 40,
			['align'] = 'RIGHT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "keystoneLevel",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_dungeon"),
			['width'] = 115,
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "keystoneDungeonId",
			["DoCellUpdate"] = ApplySettingsToCell
		},{
			['name'] = KRCLocal:Get("tooltip_week_best"),
			['width'] = 175,
			['field'] = "bestKeystoneLevel",
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			["DoCellUpdate"] = ApplySettingsToCell
		}
	}
	if supportsAltDisplay then
		cols[2]['name'] = KRCLocal:Get("tooltip_alt_of")
		cols[2]['field'] = "altOf"
	end
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.scrollTable = KeystoneRollCall.ScrollingTable:CreateST(cols,25);
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);

	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = KeystoneRollCall.listingOnClick,	
		['OnEnter'] = KeystoneRollCall.listingOnEntry,
		['OnLeave'] = KeystoneRollCall.listingOnLeave,
	});
	
	local tableWrapper = KeystoneRollCall.AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);
	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
end