local tab = {}
tab.index = 2000;
tab.isKeyTable = false;
tab.text = KRCLocal:Get("run_history");
tab.value = "run_history";
tab.propertyShowComplete = nil;
tab.propertyShowAlts = nil;
tab.propertyShowKeyless = nil;
tab.propertyShowOffline = nil;
tab.propertyShowLeveling = nil;
tab.propertyShowOpposingFaction = nil;
tab.showFilterResultsEditBox = false;

local entryBlocks = {}

local function CreateSpacer(width)
	local label = KeystoneRollCall.AceGUI:Create('Label');
	label:SetText(" ");
	if width <= 0 then
		label:SetFullWidth(true)
	else
		label:SetWidth(width)
	end
	return label
end

tab.setupContainer = function(container)
	local scrollContainer = KeystoneRollCall.AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
	scrollContainer:SetFullWidth(true)
	scrollContainer:SetHeight(398)
	scrollContainer:SetLayout("Fill") -- important!
	container:AddChild(scrollContainer)

	scrollFrame = KeystoneRollCall.AceGUI:Create("ScrollFrame")
	scrollFrame:SetLayout("Flow") -- probably?
	scrollContainer:AddChild(scrollFrame)
		
	if entryBlocks then
		for i,entryBlock in ipairs(entryBlocks) do
			entryBlock.frame:Hide()
			for key,val in pairs(entryBlock.frame) do
				if type(val) =="table" and val.Hide then
					val:Hide()
				end
			end
		end
	end
	
	entryBlocks = {}
	local lastEntry = nil;
	local runHistory = KCLib:GetPlayerKeystone().runHistory;
	for i, runInfo in ipairs(runHistory) do
		local score = KCLib:GetRunScore(runInfo.dungeonId, runInfo.level, runInfo.completionTime);
		local name, _, timeLimit = C_ChallengeMode.GetMapUIInfo(runInfo.dungeonId);
		
		local entryBlock = KeystoneRollCall.AceGUI:Create("SimpleGroup")
		entryBlocks[i] = entryBlock;
		entryBlock.index = i
		entryBlock.frame:SetMovable(false)
		entryBlock:SetFullWidth(true)
		entryBlock:SetLayout("Flow")
		entryBlock:SetHeight(120)
		
		local texture = entryBlock.frame:CreateTexture(nil, "BACKGROUND");
		texture:SetColorTexture(.5,.5,.5,0);
		texture:SetPoint("TOPLEFT",entryBlock.frame,"TOPLEFT",0,0);
		texture:SetPoint("BOTTOMRIGHT",entryBlock.frame,0,0);
		entryBlock.frame.texture = texture;
		
		local dungeonIcon = entryBlock.frame:CreateTexture(nil, "ARTWORK");
		dungeonIcon:SetPoint("LEFT",texture,"LEFT",0,0);
		dungeonIcon:SetWidth(77);
		dungeonIcon:SetHeight(77);
		dungeonIcon:SetTexture(select(4,C_ChallengeMode.GetMapUIInfo(runInfo.dungeonId)));
		entryBlock.frame.dungeonIcon = dungeonIcon;
	
		local resultBox = entryBlock.frame:CreateTexture(nil, "ARTWORK");
		resultBox:SetPoint("right",texture,"right",0,0);
		resultBox:SetWidth(77);
		resultBox:SetHeight(77);
		resultBox:SetTexture(select(4,C_ChallengeMode.GetMapUIInfo(runInfo.dungeonId)));
		entryBlock.frame.resultBox = resultBox;
	
		local resultKeyChange = entryBlock.frame:CreateFontString(nil,"ARTWORK");
		resultKeyChange:SetFont(KeystoneRollCall.db.global.primaryFontPath, 45, "OUTLINE");
		resultKeyChange:SetPoint("CENTER",resultBox,"CENTER",0,5);
		resultKeyChange:SetTextColor(1,0.8196079,0,1);
		resultKeyChange:SetJustifyH("CENTER");
		resultKeyChange:SetJustifyV("CENTER");
		entryBlock.frame.resultKeyChange = resultKeyChange;
		
		local scoreLabel = entryBlock.frame:CreateFontString()
		scoreLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16, "OUTLINE");
		scoreLabel:SetTextColor(1,0.8196079,0,1);
		scoreLabel:SetPoint("BOTTOM",resultBox,"BOTTOM",0,0);
		scoreLabel:SetJustifyH("CENTER");
		scoreLabel:SetJustifyV("BOTTOM");
		scoreLabel:SetText(score);
		entryBlock.frame.scoreLabel = scoreLabel;
	
		local levelLabel = entryBlock.frame:CreateFontString()
		levelLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 45, "OUTLINE");
		levelLabel:SetTextColor(1,0.8196079,0,1);
		levelLabel:SetPoint("CENTER",dungeonIcon,"CENTER",0,5);
		levelLabel:SetJustifyH("CENTER");
		levelLabel:SetJustifyV("CENTER");
		levelLabel:SetText(runInfo.level or 0);
		entryBlock.frame.levelLabel = levelLabel;
	
		local modifierLabel = entryBlock.frame:CreateFontString()
		modifierLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16, "OUTLINE");
		modifierLabel:SetTextColor(1,0.8196079,0,1);
		modifierLabel:SetPoint("BOTTOM",dungeonIcon,"BOTTOM",0,0);
		modifierLabel:SetJustifyH("CENTER");
		modifierLabel:SetJustifyV("BOTTOM");
		modifierLabel:SetText("+"..runInfo.enemyDamagerModifier .. "%");
		entryBlock.frame.modifierLabel = modifierLabel;
		
		local summaryLabel = entryBlock.frame:CreateFontString()
		summaryLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "OUTLINE");
		summaryLabel:SetTextColor(1,0.8196079,0,1);
		summaryLabel:SetPoint("TOP",texture,"TOP",0,-5);
		summaryLabel:SetPoint("LEFT",dungeonIcon,"RIGHT",0,0);
		summaryLabel:SetPoint("RIGHT",resultBox,"LEFT",0,0);
		summaryLabel:SetJustifyH("CENTER");
		summaryLabel:SetJustifyV("BOTTOM");
		entryBlock.frame.summaryLabel = summaryLabel;
		
		local deathsLabel = entryBlock.frame:CreateFontString()
		deathsLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "OUTLINE");
		deathsLabel:SetTextColor(1,0.8196079,0,1);
		deathsLabel:SetPoint("BOTTOM",texture,"BOTTOM",0,5);
		deathsLabel:SetPoint("LEFT",dungeonIcon,"RIGHT",5,0);
		deathsLabel:SetJustifyH("CENTER");
		deathsLabel:SetJustifyV("BOTTOM");
		deathsLabel:SetText(KRCLocal:Get("run_death_count",runInfo.deathCount, runInfo.deathTimePenalty));
		entryBlock.frame.deathsLabel = deathsLabel;
		
		local dateLabel = entryBlock.frame:CreateFontString()
		dateLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "OUTLINE");
		dateLabel:SetTextColor(1,0.8196079,0,1);
		dateLabel:SetPoint("BOTTOM",texture,"BOTTOM",0,5);
		dateLabel:SetPoint("RIGHT",resultBox,"LEFT",-5,0);
		dateLabel:SetJustifyH("CENTER");
		dateLabel:SetJustifyV("BOTTOM");
		dateLabel:SetText( date("%m/%d/%y",runInfo.timestamp));
		entryBlock.frame.dateLabel = dateLabel;
		
		if runInfo.partyMembers and #runInfo.partyMembers > 0 then
			local lastElement = nil;
			for z, partyMember in ipairs(runInfo.partyMembers) do
				local classInfo = C_CreatureInfo.GetClassInfo(partyMember.classID);

				local memberBox = entryBlock.frame:CreateTexture(nil, "ARTWORK", nil, 1);
				memberBox:SetColorTexture(.075,.075,.075,1);
				if lastElement then
					memberBox:SetPoint("LEFT",lastElement,"RIGHT",2,0);
				elseif #runInfo.partyMembers >= 5 then
					memberBox:SetPoint("LEFT",dungeonIcon,"RIGHT",4,0);
				else
					memberBox:SetPoint("LEFT",dungeonIcon,"RIGHT",58,0);
				end
				memberBox:SetWidth(103);
				memberBox:SetHeight(22);
				entryBlock.frame["memberBox"..z] = memberBox;
				lastElement = memberBox;

				local classIcon = entryBlock.frame:CreateTexture(nil, "ARTWORK", nil, 2);
				classIcon:SetPoint("TOPLEFT",memberBox,"TOPLEFT",4,-4);
				classIcon:SetSize(12,12)
				classIcon:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES");
				classIcon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classInfo.classFile])); 	
				entryBlock.frame["classIcon"..z] = classIcon;	
				
				local memberName = entryBlock.frame:CreateFontString()
				memberName:SetFont(KeystoneRollCall.db.global.primaryFontPath, 11, "OUTLINE");
				memberName:SetTextColor(GetClassColor(classInfo.classFile));
				memberName:SetPoint("TOPRIGHT",memberBox,"TOPRIGHT",-4,-4);
				memberName:SetPoint("LEFT",classIcon,"RIGHT",4,0);
				memberName:SetJustifyH("LEFT");
				memberName:SetJustifyV("CENTER");
				memberName:SetText(partyMember.name);
				entryBlock.frame["memberName"..z] = memberName;
			end
		end
		
		if i%2==1 then
			texture:SetColorTexture(.05,.05,.05,1)
		else
			texture:SetColorTexture(.1,.1,.1,1)
		end
				
		if runInfo.keystoneUpgradeLevels > 0 then
			resultBox:SetColorTexture(0,1,0,1);
			scoreLabel:SetTextColor(0.5,1,0.5,1)
			resultKeyChange:SetTextColor(0.5,1,0.5,1)
			resultKeyChange:SetText("+"..tostring(runInfo.keystoneUpgradeLevels))
		else
			resultBox:SetColorTexture(1,0,0,1);
			scoreLabel:SetTextColor(1,0.5,0.5,1)
			resultKeyChange:SetTextColor(1,0.5,0.5,1)
			resultKeyChange:SetText("-1")
		end
		summaryLabel:SetText(KRCLocal:Get("run_summary",name,runInfo.level,KeystoneRollCall:FormatTimeString(runInfo.completionTime),KeystoneRollCall:FormatTimeString(timeLimit),score))
		
		entryBlock:AddChild(CreateSpacer(0))
		entryBlock:AddChild(CreateSpacer(0))
		entryBlock:AddChild(CreateSpacer(0))
		entryBlock:AddChild(CreateSpacer(0))
		entryBlock:AddChild(CreateSpacer(0))
		entryBlock:AddChild(CreateSpacer(0))
		entryBlock.frame:Show()

		scrollFrame:AddChild(entryBlock,lastEntry);			
		lastEntry = entryBlock;
	end
	
	scrollFrame:DoLayout()
	KeystoneRollCall.frame:DoLayout()
end

KeystoneRollCall.tabs[tab.value] = tab;